@@ -26,7 +26,7 @@ url="$(svn info --show-item url)"
 branch="${url##*/}"
 export MALLOC_OPTIONS=S
 (cd .. && gmake BRANCH="$branch" PARALLEL="4" THREADING="no" JAVA="no" \
-                EXCLUSIVE_WC_LOCKS=1 \
+                EXCLUSIVE_WC_LOCKS=1 SVN_CHECK_FS_BACKENDS=fsfs \
                                   svn-check-local \
                                   svn-check-svn \
                                   svn-check-neon \
